@@ -21,8 +21,12 @@ module Milter
   class Client
     class CommandLine
       attr_reader :options, :option_parser
-      def initialize
+      attr_accessor :name
+      def initialize(options={})
+        @name = options[:name] || File.basename($0, '.*')
         setup_options
+        @option_parser = OptionParser.new(banner)
+        yield(self) if block_given?
         setup_option_parser
       end
 
@@ -35,8 +39,18 @@ module Milter
           exit(false)
         end
         client = Milter::Client.new
+        client.on_error do |_client, error|
+          Milter::Logger.error("[client][error] #{error.message}")
+        end
+        client.start_syslog(@name) if @options.syslog
         client.status_on_error = @options.status_on_error
         client.connection_spec = @options.connection_spec
+        client.effective_user = @options.user
+        client.effective_group = @options.group
+        client.unix_socket_group = @options.unix_socket_group
+        if @options.unix_socket_mode
+          client.unix_socket_mode = @options.unix_socket_mode
+        end
         yield(client, options) if block_given?
         daemonize if @options.run_as_daemon
         client.main
@@ -48,10 +62,14 @@ module Milter
         @options.connection_spec = "inet:20025"
 	@options.status_on_error = "accept"
         @options.run_as_daemon = false
+        @options.user = nil
+        @options.group = nil
+        @options.unix_socket_group = nil
+        @options.unix_socket_mode = nil
+        @options.syslog = false
       end
 
       def setup_option_parser
-        @option_parser = OptionParser.new(banner)
         setup_common_options
       end
 
@@ -74,8 +92,9 @@ module Milter
           exit(true)
         end
 
-        @option_parser.on("--version", "Show version.") do
-          puts VERSION
+        @option_parser.on("--library-version",
+                          "Show milter library version.") do
+          puts Milter::TOOLKIT_VERSION.join(".")
           exit(true)
         end
       end
@@ -85,29 +104,49 @@ module Milter
         @option_parser.separator "milter options"
 
         @option_parser.on("-s", "--connection-spec=SPEC",
-                "Specify connection spec as [SPEC].",
-                "(#{@options.connection_spec})") do |spec|
+                          "Specify connection spec as [SPEC].",
+                          "(#{@options.connection_spec})") do |spec|
           @options.connection_spec = spec
         end
 
         @option_parser.on("--[no-]daemon",
-                "Run as a daemon process.",
-                "(#{@options.run_as_daemon})") do |run_as_daemon|
+                          "Run as a daemon process.",
+                          "(#{@options.run_as_daemon})") do |run_as_daemon|
           @options.run_as_daemon = run_as_daemon
         end
 
         statuses = ["accept", "reject", "temporary_failure"]
         @option_parser.on("--status-on-error=STATUS",
-                statuses,
-                "Specify status on error.",
-                "(#{@options.status_on_error})") do |status|
+                          statuses,
+                          "Specify status on error.",
+                          "(#{@options.status_on_error})") do |status|
           @options.status_on_error = status
         end
 
-        @option_parser.on("--database-path=PATH",
-                "Specify database path to store messages as [PATH].",
-                "(#{@options.database_path})") do |path|
-          @options.database_path = path
+        @option_parser.on("--user=USER",
+                          "Run as USER's process (need root privilege)",
+                          "(#{@options.user})") do |user|
+          @options.user = user
+        end
+
+        @option_parser.on("--group=GROUP",
+                          "Run as GROUP's process (need root privilege)",
+                          "(#{@options.group})") do |gorup|
+          @options.group = group
+        end
+
+        @option_parser.on("--unix-socket-group=GROUP",
+                          "Change UNIX domain socket group to GROUP",
+                          "(#{@options.unix_socket_group})") do |gorup|
+          @options.unix_socket_group = group
+        end
+
+        client = Milter::Client.new
+        @option_parser.on("--unix-socket-mode=MODE",
+                          "Change UNIX domain socket mode to MODE",
+                          "(#{client.default_unix_socket_mode})") do |mode|
+          client.unix_socket_mode = mode
+          @options.unix_socket_mode = mode
         end
       end
 
@@ -118,9 +157,9 @@ module Milter
         level_names = Milter::LogLevelFlags.values.collect {|value| value.nick}
         level_names << "all"
         @option_parser.on("--log-level=LEVEL",
-                "Specify log level as [LEVEL].",
-                "Select from [%s]." % level_names.join(', '),
-                "(#{ENV['MILTER_LOG_LEVEL'] || 'none'})") do |level|
+                          "Specify log level as [LEVEL].",
+                          "Select from [%s]." % level_names.join(', '),
+                          "(#{ENV['MILTER_LOG_LEVEL'] || 'none'})") do |level|
           if level.empty?
             ENV["MILTER_LOG_LEVEL"] = nil
           else
@@ -128,9 +167,15 @@ module Milter
           end
         end
 
+        @option_parser.on("--[no-]syslog",
+                          "Use syslog",
+                          "(#{@options.syslog})") do |bool|
+          @options.syslog = bool
+        end
+
         @option_parser.on("--verbose",
-                "Show messages verbosely.",
-                "Alias of --log-level=all.") do
+                          "Show messages verbosely.",
+                          "Alias of --log-level=all.") do
           ENV["MILTER_LOG_LEVEL"] = "all"
         end
       end
