@@ -40,6 +40,8 @@ require 'milter/manager/redhat-detector'
 require 'milter/manager/freebsd-rc-detector'
 require 'milter/manager/pkgsrc-rc-detector'
 
+Milter::Logger.domain = "milter-manager"
+
 module Milter::Manager
   class Configuration
     def dump
@@ -896,6 +898,12 @@ module Milter::Manager
         end
       end
 
+      def report_memory_profile
+        maintained do
+          Milter::MemoryProfile.report
+        end
+      end
+
       private
       def update_location(key, reset, deep_level=2)
         full_key = "manager.#{key}"
@@ -910,6 +918,16 @@ module Milter::Manager
           message << "total:#{n_objects[:total]} "
           message << "Proc:#{n_objects[:proc]} "
           message << "GLib::Object:#{n_objects[:glib_object]}"
+          data = Milter::MemoryProfile.data
+          if data
+            n_allocates, n_zero_initializes, n_frees = data
+            n_used_in_kb = (n_allocates - n_frees) / 1024.0
+            used_percent = n_used_in_kb / memory_usage_in_kb * 100
+            message << ("  (%dKB)(%.2f%%)" % [n_used_in_kb.to_i, used_percent])
+            message << " allocated:#{n_allocates}"
+            free_percent = n_frees.to_f / n_allocates * 100
+            message << (" freed:%d(%.2f%%)" % [n_frees, free_percent])
+          end
           Milter::Logger.statistics(message)
         end
 
