@@ -23,6 +23,7 @@
 #include "rb-milter-private.h"
 
 void Init_milter_logger (void);
+void Init_milter_memory_profile (void);
 void Init_milter_socket_address (void);
 void Init_milter_utils (void);
 void Init_milter_connection (void);
@@ -34,6 +35,7 @@ void Init_milter_encoder (void);
 void Init_milter_command_encoder (void);
 void Init_milter_reply_encoder (void);
 void Init_milter_decoder (void);
+void Init_milter_error_emittable (void);
 void Init_milter_agent (void);
 void Init_milter_protocol_agent (void);
 
