@@ -27,6 +27,7 @@ Init_milter_core (void)
     rb_mMilterCore = rb_define_module_under(rb_mMilter, "Core");
 
     Init_milter_logger();
+    Init_milter_memory_profile();
     Init_milter_socket_address();
     Init_milter_utils();
     Init_milter_connection();
@@ -37,6 +38,7 @@ Init_milter_core (void)
     Init_milter_command_encoder();
     Init_milter_reply_encoder();
     Init_milter_decoder();
+    Init_milter_error_emittable();
     Init_milter_agent();
     Init_milter_protocol_agent();
 }
