@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -473,13 +473,11 @@ cb_abort (MilterClientContext *context, MilterClientContextState state,
 static void
 cb_finished (MilterFinishedEmittable *emittable, gpointer user_data)
 {
-    MilterClientContext *context;
     SmfiContext *smfi_context = user_data;
 
     if (filter_description->xxfi_close)
         filter_description->xxfi_close(smfi_context);
 
-    context = MILTER_CLIENT_CONTEXT(emittable);
     g_object_unref(smfi_context);
 }
 
@@ -595,6 +593,8 @@ smfi_main (void)
 int
 smfi_setbacklog (int backlog)
 {
+    libmilter_compatible_initialize();
+
     if (backlog <= 0)
         return MI_FAILURE;
 
@@ -629,6 +629,8 @@ smfi_setdbg (int level)
 int
 smfi_settimeout (int timeout)
 {
+    libmilter_compatible_initialize();
+
     timeout = timeout;
 
     return MI_SUCCESS;
@@ -670,6 +672,8 @@ smfi_setconn (char *spec)
 int
 smfi_stop (void)
 {
+    libmilter_compatible_initialize();
+
     if (client)
         milter_client_shutdown(client);
     return MI_SUCCESS;
@@ -679,6 +683,8 @@ int
 smfi_version (unsigned int *major, unsigned int *minor,
               unsigned int *patch_level)
 {
+    libmilter_compatible_initialize();
+
     if (major)
         *major = SM_LM_VRS_MAJOR(SMFI_VERSION);
     if (minor)
