@@ -232,7 +232,11 @@ struct _MilterClientClass
                                            socklen_t        address_size);
     gchar *(*get_default_connection_spec) (MilterClient *client);
     guint  (*get_unix_socket_mode)        (MilterClient *client);
+    void   (*set_unix_socket_mode)        (MilterClient *client,
+                                           guint         mode);
     const gchar *(*get_unix_socket_group) (MilterClient *client);
+    void   (*set_unix_socket_group)       (MilterClient *client,
+                                           const gchar  *group);
     gboolean (*is_remove_unix_socket_on_close)
                                           (MilterClient *client);
     guint  (*get_suspend_time_on_unacceptable)
@@ -404,6 +408,18 @@ void                 milter_client_set_timeout       (MilterClient  *client,
 guint                milter_client_get_unix_socket_mode
                                                      (MilterClient  *client);
 
+/**
+ * milter_client_set_unix_socket_mode:
+ * @client: a %MilterClient.
+ * @mode: the UNIX domain socket mode.
+ *
+ * Sets the UNIX domain socket mode. @mode == '0' means that
+ * the default UNIX domain socket mode will be used.
+ */
+void                 milter_client_set_unix_socket_mode
+                                                     (MilterClient  *client,
+                                                      guint          mode);
+
 /**
  * milter_client_get_default_unix_socket_mode:
  * @client: a %MilterClient.
@@ -437,6 +453,17 @@ void                 milter_client_set_default_unix_socket_mode
 const gchar         *milter_client_get_unix_socket_group
                                                      (MilterClient  *client);
 
+/**
+ * milter_client_set_unix_socket_group:
+ * @client: a %MilterClient.
+ * @group: the UNIX domain socket group.
+ *
+ * Sets the group of UNIX domain socket.
+ */
+void                 milter_client_set_unix_socket_group
+                                                     (MilterClient  *client,
+                                                      const gchar   *group);
+
 /**
  * milter_client_get_default_unix_socket_group:
  * @client: a %MilterClient.
@@ -697,6 +724,16 @@ gboolean             milter_client_daemonize         (MilterClient  *client,
 guint                milter_client_get_n_processing_sessions
                                                      (MilterClient  *client);
 
+/**
+ * milter_client_start_syslog:
+ * @client: a %MilterClient.
+ * @identify: the identify name in syslog.
+ *
+ * Starts logging by syslog with @identify name.
+ */
+void                 milter_client_start_syslog      (MilterClient  *client,
+                                                      const gchar   *identify);
+
 G_END_DECLS
 
 #endif /* __MILTER_CLIENT_H__ */
