@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -23,6 +23,7 @@
 #include <glib-object.h>
 
 #include <milter/core.h>
+#include <milter/client/milter-client-objects.h>
 
 G_BEGIN_DECLS
 
@@ -329,7 +330,6 @@ typedef enum
     (MILTER_CLIENT_CONTEXT_STATE_ENVELOPE_FROM <= (state) &&            \
      (state) < MILTER_CLIENT_CONTEXT_STATE_END_OF_MESSAGE)
 
-typedef struct _MilterClientContext         MilterClientContext;
 typedef struct _MilterClientContextClass    MilterClientContextClass;
 
 struct _MilterClientContext
@@ -403,6 +403,9 @@ struct _MilterClientContextClass
     void         (*abort_response)     (MilterClientContext *context,
                                         MilterStatus         status);
     void         (*timeout)            (MilterClientContext *context);
+
+    void         (*message_processed)  (MilterClientContext *context,
+                                        MilterMessageResult *result);
 };
 
 GQuark               milter_client_context_error_quark       (void);
@@ -411,6 +414,7 @@ GType                milter_client_context_get_type          (void) G_GNUC_CONST
 
 /**
  * milter_client_context_new:
+ * @client: a %MilterClient for the context.
  *
  * Creates a new context object. Normally, context object is
  * created by %MilterClient and passed by
@@ -418,7 +422,7 @@ GType                milter_client_context_get_type          (void) G_GNUC_CONST
  *
  * Returns: a new %MilterClientContext object.
  */
-MilterClientContext *milter_client_context_new               (void);
+MilterClientContext *milter_client_context_new               (MilterClient *client);
 
 /**
  * milter_client_context_feed:
@@ -819,7 +823,71 @@ void                 milter_client_context_set_option        (MilterClientContex
  */
 MilterOption        *milter_client_context_get_option        (MilterClientContext *context);
 
+/**
+ * milter_client_context_set_socket_address:
+ * @context: a %MilterClientContext.
+ * @address: a %MilterGenericSocketAddress.
+ *
+ * Sets the socket address of connected server for the context.
+ */
+void                 milter_client_context_set_socket_address(MilterClientContext *context,
+                                                              MilterGenericSocketAddress *address);
 
+/**
+ * milter_client_context_get_option:
+ * @context: a %MilterClientContext.
+ *
+ * Gets the socket address of connected server for the context.
+ *
+ * Returns: the socket address of connected server for the context.
+ */
+MilterGenericSocketAddress *milter_client_context_get_socket_address
+                                                             (MilterClientContext *context);
+
+/**
+ * milter_client_context_get_message_result:
+ * @context: a %MilterClientContext.
+ *
+ * Gets the message result of @context.
+ *
+ * Returns: the message result of @context.
+ */
+MilterMessageResult *milter_client_context_get_message_result
+                                                       (MilterClientContext *context);
+
+
+/**
+ * milter_client_context_set_message_result:
+ * @context: a %MilterClientContext.
+ * @result: the message result.
+ *
+ * Sets the message result of @context.
+ */
+void                 milter_client_context_set_message_result
+                                                       (MilterClientContext *context,
+                                                        MilterMessageResult *result);
+
+/**
+ * milter_client_context_reset_message_related_data:
+ * @context: a %MilterClientContext.
+ *
+ * Resets message related data of @context.
+ *
+ * Normally, you don't need to call this function.
+ */
+void                 milter_client_context_reset_message_related_data
+                                                       (MilterClientContext *context);
+
+/**
+ * milter_client_context_get_n_processing_sessions:
+ * @context: a %MilterClientContext.
+ *
+ * Returns number of the current processing sessions.
+ *
+ * Returns: number of the current processing sessions.
+ */
+guint                milter_client_context_get_n_processing_sessions
+                                                       (MilterClientContext  *context);
 
 G_END_DECLS
 
