@@ -96,7 +96,9 @@ struct _MilterClientPrivate
     gint listen_backlog;
     GMutex *quit_mutex;
     gboolean quitting;
+    guint unix_socket_mode;
     guint default_unix_socket_mode;
+    gchar *unix_socket_group;
     gchar *default_unix_socket_group;
     gboolean default_remove_unix_socket_on_close;
     gboolean remove_unix_socket_on_create;
@@ -111,6 +113,8 @@ struct _MilterClientPrivate
 
     guint finisher_id;
     GList *finished_data;
+
+    MilterSyslogLogger *syslog_logger;
 };
 
 typedef struct _MilterClientProcessData
@@ -226,7 +230,9 @@ _milter_client_init (MilterClient *client)
     priv->listen_backlog = -1;
     priv->quit_mutex = g_mutex_new();
     priv->quitting = FALSE;
+    priv->unix_socket_mode = 0;
     priv->default_unix_socket_mode = 0660;
+    priv->unix_socket_group = NULL;
     priv->default_unix_socket_group = NULL;
     priv->default_remove_unix_socket_on_close = TRUE;
     priv->remove_unix_socket_on_create = TRUE;
@@ -242,6 +248,8 @@ _milter_client_init (MilterClient *client)
 
     priv->finisher_id = 0;
     priv->finished_data = NULL;
+
+    priv->syslog_logger = NULL;
 }
 
 static void
@@ -446,6 +454,11 @@ dispose (GObject *object)
         priv->quit_mutex = NULL;
     }
 
+    if (priv->unix_socket_group) {
+        g_free(priv->unix_socket_group);
+        priv->unix_socket_group = NULL;
+    }
+
     if (priv->default_unix_socket_group) {
         g_free(priv->default_unix_socket_group);
         priv->default_unix_socket_group = NULL;
@@ -471,6 +484,11 @@ dispose (GObject *object)
     dispose_finisher(priv);
     dispose_finished_data(MILTER_CLIENT(object));
 
+    if (priv->syslog_logger) {
+        g_object_unref(priv->syslog_logger);
+        priv->syslog_logger = NULL;
+    }
+
     G_OBJECT_CLASS(_milter_client_parent_class)->dispose(object);
 }
 
@@ -1524,10 +1542,34 @@ milter_client_get_unix_socket_mode (MilterClient *client)
     MilterClientClass *klass;
 
     klass = MILTER_CLIENT_GET_CLASS(client);
-    if (klass->get_unix_socket_mode)
+    if (klass->get_unix_socket_mode) {
         return klass->get_unix_socket_mode(client);
-    else
-        return milter_client_get_default_unix_socket_mode(client);
+    } else {
+        MilterClientPrivate *priv;
+
+        priv = MILTER_CLIENT_GET_PRIVATE(client);
+        if (priv->unix_socket_mode != 0) {
+            return priv->unix_socket_mode;
+        } else {
+            return milter_client_get_default_unix_socket_mode(client);
+        }
+    }
+}
+
+void
+milter_client_set_unix_socket_mode (MilterClient *client, guint mode)
+{
+    MilterClientClass *klass;
+
+    klass = MILTER_CLIENT_GET_CLASS(client);
+    if (klass->set_unix_socket_mode) {
+        klass->set_unix_socket_mode(client, mode);
+    } else {
+        MilterClientPrivate *priv;
+
+        priv = MILTER_CLIENT_GET_PRIVATE(client);
+        priv->unix_socket_mode = mode;
+    }
 }
 
 guint
@@ -1548,10 +1590,35 @@ milter_client_get_unix_socket_group (MilterClient *client)
     MilterClientClass *klass;
 
     klass = MILTER_CLIENT_GET_CLASS(client);
-    if (klass->get_unix_socket_group)
+    if (klass->get_unix_socket_group) {
         return klass->get_unix_socket_group(client);
-    else
-        return milter_client_get_default_unix_socket_group(client);
+    } else {
+        MilterClientPrivate *priv;
+
+        priv = MILTER_CLIENT_GET_PRIVATE(client);
+        if (priv->unix_socket_group) {
+            return priv->unix_socket_group;
+        } else {
+            return milter_client_get_default_unix_socket_group(client);
+        }
+    }
+}
+
+void
+milter_client_set_unix_socket_group (MilterClient *client, const gchar *group)
+{
+    MilterClientClass *klass;
+
+    klass = MILTER_CLIENT_GET_CLASS(client);
+    if (klass->set_unix_socket_group) {
+        klass->set_unix_socket_group(client, group);
+    } else {
+        MilterClientPrivate *priv;
+
+        priv = MILTER_CLIENT_GET_PRIVATE(client);
+        g_free(priv->unix_socket_group);
+        priv->unix_socket_group = g_strdup(group);
+    }
 }
 
 const gchar *
@@ -1750,6 +1817,19 @@ milter_client_get_n_processing_sessions (MilterClient *client)
     return MILTER_CLIENT_GET_PRIVATE(client)->n_processing_sessions;
 }
 
+void
+milter_client_start_syslog (MilterClient *client, const gchar *identify)
+{
+    MilterClientPrivate *priv;
+
+    priv = MILTER_CLIENT_GET_PRIVATE(client);
+
+    if (priv->syslog_logger) {
+        g_object_unref(priv->syslog_logger);
+    }
+    priv->syslog_logger = milter_syslog_logger_new(identify);
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
