@@ -26,6 +26,7 @@
 #include "milter-esmtp.h"
 #include "milter-message-result.h"
 #include "milter-session-result.h"
+#include "milter-memory-profile.h"
 
 /* enumerations from "milter-protocol.h" */
 GType
@@ -322,6 +323,7 @@ milter_log_level_flags_get_type (void)
       { MILTER_LOG_LEVEL_INFO, "MILTER_LOG_LEVEL_INFO", "info" },
       { MILTER_LOG_LEVEL_DEBUG, "MILTER_LOG_LEVEL_DEBUG", "debug" },
       { MILTER_LOG_LEVEL_STATISTICS, "MILTER_LOG_LEVEL_STATISTICS", "statistics" },
+      { MILTER_LOG_LEVEL_PROFILE, "MILTER_LOG_LEVEL_PROFILE", "profile" },
       { 0, NULL, NULL }
     };
     etype = g_flags_register_static ("MilterLogLevelFlags", values);
