@@ -145,6 +145,12 @@ gint             milter_utils_strcmp0        (const gchar *str1,
 
 gboolean         milter_utils_detach_io      (gchar **message);
 
+gboolean         milter_utils_guess_console_color_usability
+                                             (void);
+gboolean         milter_utils_parse_file_mode(const gchar  *string,
+                                              guint        *mode,
+                                              gchar       **error_message);
+
 G_END_DECLS
 
 #endif /* __MILTER_UTILS_H__ */
