@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -39,7 +39,8 @@ G_BEGIN_DECLS
 typedef enum
 {
     MILTER_WRITER_ERROR_NO_CHANNEL,
-    MILTER_WRITER_ERROR_IO_ERROR
+    MILTER_WRITER_ERROR_IO_ERROR,
+    MILTER_WRITER_ERROR_NOT_READY
 } MilterWriterError;
 
 typedef struct _MilterWriter         MilterWriter;
@@ -69,7 +70,8 @@ gboolean         milter_writer_write          (MilterWriter     *writer,
 gboolean         milter_writer_flush          (MilterWriter     *writer,
                                                GError          **error);
 
-void             milter_writer_start          (MilterWriter     *writer);
+void             milter_writer_start          (MilterWriter     *writer,
+                                               GMainContext     *context);
 gboolean         milter_writer_is_watching    (MilterWriter     *writer);
 void             milter_writer_shutdown       (MilterWriter     *writer);
 
