@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -29,10 +29,12 @@
 
 #include "milter-manager-module.h"
 #include "milter-manager-configuration.h"
+#include "milter-manager-leader.h"
 #include "milter-manager-children.h"
 #include <milter/core/milter-marshalers.h>
 
-#define DEFAULT_MAINTENANCE_INTERVAL 100
+#define DEFAULT_MAINTENANCE_INTERVAL 10
+#define DEFAULT_CONNECTION_CHECK_INTERVAL 0
 
 #define MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(obj)                   \
     (G_TYPE_INSTANCE_GET_PRIVATE((obj),                                 \
@@ -67,8 +69,9 @@ struct _MilterManagerConfigurationPrivate
     guint suspend_time_on_unacceptable;
     guint max_connections;
     guint max_file_descriptors;
-    guint processed_sessions;
     gchar *custom_configuration_directory;
+    GHashTable *locations;
+    guint connection_check_interval;
 };
 
 enum
@@ -96,12 +99,13 @@ enum
     PROP_SUSPEND_TIME_ON_UNACCEPTABLE,
     PROP_MAX_CONNECTIONS,
     PROP_MAX_FILE_DESCRIPTORS,
-    PROP_PROCESSED_SESSIONS,
-    PROP_CUSTOM_CONFIGURATION_DIRECTORY
+    PROP_CUSTOM_CONFIGURATION_DIRECTORY,
+    PROP_CONNECTION_CHECK_INTERVAL
 };
 
 enum
 {
+    CONNECTED,
     TO_XML,
     LAST_SIGNAL,
 };
@@ -352,6 +356,29 @@ milter_manager_configuration_class_init (MilterManagerConfigurationClass *klass)
                                     PROP_CUSTOM_CONFIGURATION_DIRECTORY,
                                     spec);
 
+    spec = g_param_spec_uint("connection-check-interval",
+                             "Connection check interval",
+                             "The interval in seconds for checking whether "
+                             "SMTP connection between SMTP client and server "
+                             "is still connected or disconnected from "
+                             "SMTP client.",
+                             0,
+                             G_MAXUINT,
+                             DEFAULT_CONNECTION_CHECK_INTERVAL,
+                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+    g_object_class_install_property(gobject_class,
+                                    PROP_CONNECTION_CHECK_INTERVAL,
+                                    spec);
+
+
+    signals[CONNECTED] =
+        g_signal_new("connected",
+                     G_TYPE_FROM_CLASS(klass),
+                     G_SIGNAL_RUN_LAST,
+                     G_STRUCT_OFFSET(MilterManagerConfigurationClass, connected),
+                     NULL, NULL,
+                     g_cclosure_marshal_VOID__OBJECT,
+                     G_TYPE_NONE, 1, MILTER_TYPE_MANAGER_LEADER);
 
     signals[TO_XML] =
         g_signal_new("to-xml",
@@ -387,8 +414,12 @@ milter_manager_configuration_init (MilterManagerConfiguration *configuration)
     priv->max_connections =
         MILTER_CLIENT_DEFAULT_MAX_CONNECTIONS;
     priv->max_file_descriptors = 0;
-    priv->processed_sessions = 0;
     priv->custom_configuration_directory = NULL;
+    priv->locations = g_hash_table_new_full(g_str_hash,
+                                            g_str_equal,
+                                            g_free,
+                                            (GDestroyNotify)g_dataset_destroy);
+    priv->connection_check_interval = DEFAULT_CONNECTION_CHECK_INTERVAL;
 
     config_dir_env = g_getenv("MILTER_MANAGER_CONFIG_DIR");
     if (config_dir_env)
@@ -417,6 +448,11 @@ dispose (GObject *object)
         priv->manager_connection_spec = NULL;
     }
 
+    if (priv->locations) {
+        g_hash_table_unref(priv->locations);
+        priv->locations = NULL;
+    }
+
     G_OBJECT_CLASS(milter_manager_configuration_parent_class)->dispose(object);
 }
 
@@ -522,6 +558,10 @@ set_property (GObject      *object,
         milter_manager_configuration_set_custom_configuration_directory(
             config, g_value_get_string(value));
         break;
+    case PROP_CONNECTION_CHECK_INTERVAL:
+        milter_manager_configuration_set_connection_check_interval(
+            config, g_value_get_uint(value));
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -607,6 +647,9 @@ get_property (GObject    *object,
     case PROP_CUSTOM_CONFIGURATION_DIRECTORY:
         g_value_set_string(value, priv->custom_configuration_directory);
         break;
+    case PROP_CONNECTION_CHECK_INTERVAL:
+        g_value_set_uint(value, priv->connection_check_interval);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -1333,6 +1376,44 @@ milter_manager_configuration_set_custom_configuration_directory (MilterManagerCo
     priv->custom_configuration_directory = g_strdup(custom_configuration_directory);
 }
 
+static gboolean
+cb_connected_dummy (MilterManagerConfiguration *configuration,
+                    MilterManagerLeader *leader,
+                    gpointer user_data)
+{
+    return TRUE;
+}
+
+void
+milter_manager_configuration_clear_signal_handlers (MilterManagerConfiguration *configuration)
+{
+    MilterManagerConfigurationPrivate *priv;
+    gulong i, max_id;
+    gboolean handler_id_overflowed;
+    static gulong previous_max_id = 0;
+
+    priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
+
+    max_id = g_signal_connect(configuration, "connected",
+                              G_CALLBACK(cb_connected_dummy), NULL);
+
+    handler_id_overflowed = previous_max_id > max_id;
+    if (handler_id_overflowed) {
+        for (i = previous_max_id + 1; i <= G_MAXULONG; i++) {
+            if (g_signal_handler_is_connected(configuration, i)) {
+                g_signal_handler_disconnect(configuration, i);
+            }
+        }
+    }
+    previous_max_id = max_id;
+
+    for (i = 0; i <= max_id; i++) {
+        if (g_signal_handler_is_connected(configuration, i)) {
+            g_signal_handler_disconnect(configuration, i);
+        }
+    }
+}
+
 void
 milter_manager_configuration_add_egg (MilterManagerConfiguration *configuration,
                                       MilterManagerEgg         *egg)
@@ -1416,14 +1497,14 @@ milter_manager_configuration_clear_eggs (MilterManagerConfiguration *configurati
     }
 }
 
-MilterManagerChildren *
-milter_manager_configuration_create_children (MilterManagerConfiguration *configuration)
+void
+milter_manager_configuration_setup_children (MilterManagerConfiguration *configuration,
+                                             MilterManagerChildren *children,
+                                             MilterClientContext *context)
 {
     GList *node;
-    MilterManagerChildren *children;
     MilterManagerConfigurationPrivate *priv;
 
-    children = milter_manager_children_new(configuration);
     priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
 
     for (node = priv->eggs; node; node = g_list_next(node)) {
@@ -1436,12 +1517,12 @@ milter_manager_configuration_create_children (MilterManagerConfiguration *config
         child = milter_manager_egg_hatch(egg);
         if (child) {
             milter_manager_children_add_child(children, child);
-            milter_manager_egg_attach_applicable_conditions(egg, child, children);
+            milter_manager_egg_attach_applicable_conditions(egg,
+                                                            child, children,
+                                                            context);
             g_object_unref(child);
         }
     }
-
-    return children;
 }
 
 MilterStatus
@@ -1595,21 +1676,23 @@ milter_manager_configuration_clear_applicable_conditions (MilterManagerConfigura
     }
 }
 
-void
-milter_manager_configuration_clear (MilterManagerConfiguration *configuration)
+static void
+clear_controller (MilterManagerConfigurationPrivate *priv)
 {
-    MilterManagerConfigurationPrivate *priv;
-
-    priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
-
-    milter_manager_configuration_clear_eggs(configuration);
-    milter_manager_configuration_clear_applicable_conditions(configuration);
-
     if (priv->controller_connection_spec) {
         g_free(priv->controller_connection_spec);
         priv->controller_connection_spec = NULL;
     }
 
+    if (priv->controller_unix_socket_group) {
+        g_free(priv->controller_unix_socket_group);
+        priv->controller_unix_socket_group = NULL;
+    }
+}
+
+static void
+clear_manager (MilterManagerConfigurationPrivate *priv)
+{
     if (priv->manager_connection_spec) {
         g_free(priv->manager_connection_spec);
         priv->manager_connection_spec = NULL;
@@ -1617,6 +1700,20 @@ milter_manager_configuration_clear (MilterManagerConfiguration *configuration)
     priv->manager_connection_spec =
         g_strdup(MILTER_MANAGER_DEFAULT_CONNECTION_SPEC);
 
+    priv->privilege_mode = FALSE;
+    priv->daemon = FALSE;
+    priv->fallback_status = MILTER_STATUS_ACCEPT;
+    priv->maintenance_interval = DEFAULT_MAINTENANCE_INTERVAL;
+    priv->suspend_time_on_unacceptable =
+        MILTER_CLIENT_DEFAULT_SUSPEND_TIME_ON_UNACCEPTABLE;
+    priv->max_connections = 0;
+    priv->max_file_descriptors = 0;
+    priv->connection_check_interval = DEFAULT_CONNECTION_CHECK_INTERVAL;
+}
+
+static void
+clear_package (MilterManagerConfigurationPrivate *priv)
+{
     if (priv->package_platform) {
         g_free(priv->package_platform);
         priv->package_platform = g_strdup(MILTER_MANAGER_PACKAGE_PLATFORM);
@@ -1626,7 +1723,11 @@ milter_manager_configuration_clear (MilterManagerConfiguration *configuration)
         g_free(priv->package_options);
         priv->package_options = g_strdup(MILTER_MANAGER_PACKAGE_OPTIONS);
     }
+}
 
+static void
+clear_account (MilterManagerConfigurationPrivate *priv)
+{
     if (priv->effective_user) {
         g_free(priv->effective_user);
         priv->effective_user = NULL;
@@ -1638,7 +1739,33 @@ milter_manager_configuration_clear (MilterManagerConfiguration *configuration)
         priv->effective_group = NULL;
     }
     priv->effective_group = g_strdup(MILTER_MANAGER_DEFAULT_EFFECTIVE_GROUP);
+}
 
+static void
+clear_process (MilterManagerConfigurationPrivate *priv)
+{
+    if (priv->pid_file) {
+        g_free(priv->pid_file);
+        priv->pid_file = NULL;
+    }
+    priv->pid_file = g_strdup(MILTER_MANAGER_DEFAULT_PID_FILE);
+}
+
+static void
+clear_configuration (MilterManagerConfigurationPrivate *priv)
+{
+    if (priv->custom_configuration_directory) {
+        g_free(priv->custom_configuration_directory);
+        priv->custom_configuration_directory = NULL;
+    }
+
+    if (priv->locations)
+        g_hash_table_remove_all(priv->locations);
+}
+
+static void
+clear_socket (MilterManagerConfigurationPrivate *priv)
+{
     if (priv->manager_unix_socket_group) {
         g_free(priv->manager_unix_socket_group);
         priv->manager_unix_socket_group = NULL;
@@ -1646,56 +1773,60 @@ milter_manager_configuration_clear (MilterManagerConfiguration *configuration)
     priv->manager_unix_socket_group =
         g_strdup(MILTER_MANAGER_DEFAULT_SOCKET_GROUP);
 
-    if (priv->controller_unix_socket_group) {
-        g_free(priv->controller_unix_socket_group);
-        priv->controller_unix_socket_group = NULL;
-    }
-
-    if (priv->pid_file) {
-        g_free(priv->pid_file);
-        priv->pid_file = NULL;
-    }
-    priv->pid_file = g_strdup(MILTER_MANAGER_DEFAULT_PID_FILE);
-
-    if (priv->custom_configuration_directory) {
-        g_free(priv->custom_configuration_directory);
-        priv->custom_configuration_directory = NULL;
-    }
-
-    priv->privilege_mode = FALSE;
     priv->manager_unix_socket_mode = 0660;
     priv->controller_unix_socket_mode = 0660;
     priv->remove_manager_unix_socket_on_close = TRUE;
     priv->remove_controller_unix_socket_on_close = TRUE;
     priv->remove_manager_unix_socket_on_create = TRUE;
     priv->remove_controller_unix_socket_on_create = TRUE;
-    priv->daemon = FALSE;
-    priv->fallback_status = MILTER_STATUS_ACCEPT;
-    priv->maintenance_interval = DEFAULT_MAINTENANCE_INTERVAL;
-    priv->suspend_time_on_unacceptable =
-        MILTER_CLIENT_DEFAULT_SUSPEND_TIME_ON_UNACCEPTABLE;
-    priv->max_connections = 0;
-    priv->max_file_descriptors = 0;
 }
 
 void
-milter_manager_configuration_session_finished (MilterManagerConfiguration *configuration)
+milter_manager_configuration_clear (MilterManagerConfiguration *configuration)
 {
     MilterManagerConfigurationPrivate *priv;
     MilterManagerConfigurationClass *configuration_class;
 
     priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
-    priv->processed_sessions++;
 
-    if (priv->maintenance_interval == 0)
-        return;
-
-    if ((priv->processed_sessions % priv->maintenance_interval) != 0)
-        return;
+    milter_manager_configuration_clear_signal_handlers(configuration);
+    milter_manager_configuration_clear_eggs(configuration);
+    milter_manager_configuration_clear_applicable_conditions(configuration);
+    clear_controller(priv);
+    clear_manager(priv);
+    clear_package(priv);
+    clear_account(priv);
+    clear_process(priv);
+    clear_configuration(priv);
+    clear_socket(priv);
 
     configuration_class = MILTER_MANAGER_CONFIGURATION_GET_CLASS(configuration);
-    if (configuration_class->maintain)
-        configuration_class->maintain(configuration);
+    if (configuration_class->clear) {
+        GError *error = NULL;
+
+        milter_debug("[configuration][clear]");
+        if (!configuration_class->clear(configuration, &error)) {
+            milter_error("[configuration][error][clear] %s", error->message);
+            g_error_free(error);
+        }
+    }
+}
+
+void
+milter_manager_configuration_maintain (MilterManagerConfiguration *configuration)
+{
+    MilterManagerConfigurationClass *configuration_class;
+
+    configuration_class = MILTER_MANAGER_CONFIGURATION_GET_CLASS(configuration);
+    if (configuration_class->maintain) {
+        GError *error = NULL;
+
+        milter_debug("[configuration][maintain]");
+        if (!configuration_class->maintain(configuration, &error)) {
+            milter_error("[configuration][error][maintain] %s", error->message);
+            g_error_free(error);
+        }
+    }
 }
 
 gchar *
@@ -1726,7 +1857,7 @@ milter_manager_configuration_to_xml_string (MilterManagerConfiguration *configur
         if (g_stat(full_path, &status) == 0) {
             gchar *content;
 
-            content = g_strdup_printf("%ld", status.st_mtime);
+            content = g_strdup_printf("%" G_GSSIZE_FORMAT, status.st_mtime);
             milter_utils_xml_append_text_element(string,
                                                  "last-modified", content,
                                                  indent + 2);
@@ -1796,6 +1927,80 @@ milter_manager_configuration_dump (MilterManagerConfiguration *configuration)
     }
 }
 
+void
+milter_manager_configuration_set_location (MilterManagerConfiguration *configuration,
+                                           const gchar *key,
+                                           const gchar *file,
+                                           gint line)
+{
+    MilterManagerConfigurationPrivate *priv;
+    gconstpointer location;
+
+    priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
+    if (!priv->locations)
+        return;
+
+    if (!file) {
+        g_hash_table_remove(priv->locations, key);
+        return;
+    }
+
+    location = g_hash_table_lookup(priv->locations, key);
+    if (!location) {
+        gchar *duped_key;
+        duped_key = g_strdup(key);
+        location = duped_key;
+        g_hash_table_insert(priv->locations, duped_key, (gpointer)location);
+    }
+    g_dataset_set_data_full(location, "file", g_strdup(file), g_free);
+    g_dataset_set_data(location, "line", GINT_TO_POINTER(line));
+}
+
+void
+milter_manager_configuration_reset_location (MilterManagerConfiguration *configuration,
+                                             const gchar *key)
+{
+    milter_manager_configuration_set_location(configuration, key, NULL, -1);
+}
+
+gconstpointer
+milter_manager_configuration_get_location (MilterManagerConfiguration *configuration,
+                                           const gchar *key)
+{
+    MilterManagerConfigurationPrivate *priv;
+
+    priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
+    if (!priv->locations)
+        return NULL;
+
+    return g_hash_table_lookup(priv->locations, key);
+}
+
+GHashTable *
+milter_manager_configuration_get_locations (MilterManagerConfiguration *configuration)
+{
+    return MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration)->locations;
+}
+
+guint
+milter_manager_configuration_get_connection_check_interval (MilterManagerConfiguration *configuration)
+{
+    MilterManagerConfigurationPrivate *priv;
+
+    priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
+    return priv->connection_check_interval;
+}
+
+void
+milter_manager_configuration_set_connection_check_interval (MilterManagerConfiguration *configuration,
+                                                            guint                       interval_in_seconds)
+{
+    MilterManagerConfigurationPrivate *priv;
+
+    priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
+    priv->connection_check_interval = interval_in_seconds;
+}
+
 
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
