@@ -229,7 +229,7 @@ cb_context_reader_finished (MilterReader *reader, gpointer data)
 {
     MilterManagerControllerContext *context = data;
 
-    g_idle_add(cb_free_context, context);
+    g_idle_add_full(G_PRIORITY_DEFAULT, cb_free_context, context, NULL);
 }
 
 static void
@@ -255,7 +255,8 @@ process_connection(MilterManagerController *controller,
     g_signal_connect(reader, "finished",
                      G_CALLBACK(cb_context_reader_finished), context);
 
-    milter_agent_start(MILTER_AGENT(context));
+    /* FIXME: correct GMainContext */
+    milter_agent_start(MILTER_AGENT(context), NULL);
 }
 
 static gboolean
@@ -264,7 +265,6 @@ accept_connection (gint controller_fd, MilterManagerController *controller)
     gint agent_fd;
     MilterGenericSocketAddress address;
     socklen_t address_size;
-    gchar *spec;
     GIOChannel *agent_channel;
 
     address_size = sizeof(address);
@@ -276,9 +276,12 @@ accept_connection (gint controller_fd, MilterManagerController *controller)
         return TRUE;
     }
 
-    spec = milter_connection_address_to_spec(&(address.address.base));
-    milter_debug("[controller][accept] %d: %s", agent_fd, spec);
-    g_free(spec);
+    if (milter_need_debug_log()) {
+        gchar *spec;
+        spec = milter_connection_address_to_spec(&(address.address.base));
+        milter_debug("[controller][accept] %d: %s", agent_fd, spec);
+        g_free(spec);
+    }
 
     agent_channel = g_io_channel_unix_new(agent_fd);
     g_io_channel_set_encoding(agent_channel, NULL, NULL);
