@@ -254,19 +254,27 @@ shutdown_client (int signum)
     }
 }
 
-static void
-reload_configuration (int signum)
+static gboolean
+cb_idle_reload_configuration (gpointer user_data)
 {
-    MilterManagerConfiguration *config;
+    if (the_manager) {
+        MilterManagerConfiguration *config;
 
-    if (!the_manager)
-        return;
+        config = milter_manager_get_configuration(the_manager);
+        if (config)
+            milter_manager_configuration_reload(config);
+    }
 
-    config = milter_manager_get_configuration(the_manager);
-    if (!config)
-        return;
+    return FALSE;
+}
 
-    milter_manager_configuration_reload(config);
+static void
+reload_configuration_request (int signum)
+{
+    g_idle_add_full(G_PRIORITY_DEFAULT,
+                    cb_idle_reload_configuration,
+                    NULL,
+                    NULL);
 }
 
 static void
@@ -361,12 +369,16 @@ prepare_process_launcher_pipes_for_manager (gint *command_pipe,
 }
 
 static void
-start_process_launcher (GIOChannel *read_channel, GIOChannel *write_channel)
+start_process_launcher (GIOChannel *read_channel, GIOChannel *write_channel,
+                        gboolean daemon)
 {
     MilterManagerProcessLauncher *launcher;
     MilterReader *reader;
     MilterWriter *writer;
     gchar *error_message = NULL;
+    MilterSyslogLogger *logger;
+
+    logger = milter_syslog_logger_new("milter-manager-process-launcher");
 
     reader = milter_reader_io_channel_new(read_channel);
     g_io_channel_unref(read_channel);
@@ -381,7 +393,7 @@ start_process_launcher (GIOChannel *read_channel, GIOChannel *write_channel)
 
     milter_agent_start(MILTER_AGENT(launcher), NULL);
 
-    if (!milter_utils_detach_io(&error_message)) {
+    if (daemon && !milter_utils_detach_io(&error_message)) {
         milter_error("[process-launcher][error] failed to detach IO: %s",
                      error_message);
         g_free(error_message);
@@ -390,6 +402,8 @@ start_process_launcher (GIOChannel *read_channel, GIOChannel *write_channel)
     milter_manager_process_launcher_run(launcher);
 
     g_object_unref(launcher);
+
+    g_object_unref(logger);
 }
 
 static gboolean
@@ -401,6 +415,8 @@ start_process_launcher_process (MilterManager *manager)
     gint *reply_pipe_p;
     GIOChannel *read_channel = NULL;
     GIOChannel *write_channel = NULL;
+    MilterManagerConfiguration *config;
+    gboolean daemon;
 
     command_pipe_p = command_pipe;
     reply_pipe_p = reply_pipe;
@@ -413,12 +429,14 @@ start_process_launcher_process (MilterManager *manager)
            'milter-manager: process-launcher' by
            setproctitle() on *BSD, memcpy() argv on Linux or
            spawning another process. */
+        config = milter_manager_get_configuration(manager);
+        daemon = milter_manager_configuration_is_daemon(config);
         g_object_unref(manager);
         prepare_process_launcher_pipes_for_process_launcher(command_pipe_p,
                                                             reply_pipe_p,
                                                             &read_channel,
                                                             &write_channel);
-        start_process_launcher(read_channel, write_channel);
+        start_process_launcher(read_channel, write_channel, daemon);
         _exit(EXIT_FAILURE);
         break;
     case -1:
@@ -643,7 +661,7 @@ milter_manager_main (void)
     GError *error = NULL;
     struct sigaction report_stack_trace_action;
     struct sigaction shutdown_client_action;
-    struct sigaction reload_configuration_action;
+    struct sigaction reload_configuration_request_action;
 
     config = milter_manager_configuration_new(NULL);
     if (option_config_dir)
@@ -702,8 +720,13 @@ milter_manager_main (void)
     }
 
     controller = milter_manager_controller_new(manager);
-    if (controller)
-        milter_manager_controller_listen(controller, NULL);
+    if (controller && !milter_manager_controller_listen(controller, &error)) {
+        milter_manager_error("failed to listen controller socket: %s",
+                             error->message);
+        g_error_free(error);
+        g_object_unref(controller);
+        controller = NULL;
+    }
 
     daemon = milter_manager_configuration_is_daemon(config);
     if (daemon) {
@@ -712,6 +735,8 @@ milter_manager_main (void)
         } else {
             milter_manager_error("failed to daemonize: %s", error->message);
             g_error_free(error);
+            if (controller)
+                g_object_unref(controller);
             g_object_unref(manager);
             return FALSE;
         }
@@ -741,7 +766,7 @@ milter_manager_main (void)
 
     SETUP_SIGNAL_ACTION(report_stack_trace);
     SETUP_SIGNAL_ACTION(shutdown_client);
-    SETUP_SIGNAL_ACTION(reload_configuration);
+    SETUP_SIGNAL_ACTION(reload_configuration_request);
 #undef SETUP_SIGNAL_ACTION
 
 #define SET_SIGNAL_ACTION(SIGNAL, signal, action)               \
@@ -754,7 +779,7 @@ milter_manager_main (void)
     SET_SIGNAL_ACTION(ABRT, abort, report_stack_trace_action);
     SET_SIGNAL_ACTION(INT, int, shutdown_client_action);
     SET_SIGNAL_ACTION(TERM, term, shutdown_client_action);
-    SET_SIGNAL_ACTION(HUP, hup, reload_configuration_action);
+    SET_SIGNAL_ACTION(HUP, hup, reload_configuration_request_action);
 #undef SET_SIGNAL_ACTION
 
     if (!milter_client_main(client))
