@@ -336,7 +336,7 @@ create_connect_packet_ipv4 (void)
     port = g_htons(50443);
     address.sin_family = AF_INET;
     address.sin_port = port;
-    inet_aton(ip_address, &(address.sin_addr));
+    inet_pton(AF_INET, ip_address, &(address.sin_addr));
 
     packet_free();
     milter_command_encoder_encode_connect(encoder,
