@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -115,8 +115,9 @@ sockaddr_in_new (guint16 port, const gchar *host)
     struct in_addr ip_address;
 
     if (host) {
-        if (inet_aton(host, &ip_address) == 0)
+        if (inet_pton(AF_INET, host, &ip_address) == 0) {
             cut_error("invalid host address: <%s>", host);
+        }
     } else {
         ip_address.s_addr = INADDR_ANY;
     }
@@ -242,7 +243,8 @@ data_parse_connection_spec_unix (void)
                                    MILTER_CONNECTION_ERROR,
                                    MILTER_CONNECTION_ERROR_INVALID_FORMAT,
                                    "spec doesn't have content: <local:>")),
-                 test_data_free);
+                 test_data_free,
+                 NULL);
 }
 
 void
@@ -304,7 +306,8 @@ data_parse_connection_spec_inet (void)
                                    MILTER_CONNECTION_ERROR,
                                    MILTER_CONNECTION_ERROR_INVALID_FORMAT,
                                    "spec doesn't have content: <inet:>")),
-                 test_data_free);
+                 test_data_free,
+                 NULL);
 }
 
 void
@@ -313,6 +316,8 @@ test_parse_connection_spec_inet (gconstpointer data)
     const TestData *test_data = data;
     gint domain;
     struct sockaddr_in *expected_in, *actual_in;
+    gchar expected_address_string[INET6_ADDRSTRLEN];
+    gchar actual_address_string[INET6_ADDRSTRLEN];
     GError *error = NULL;
 
     milter_connection_parse_spec(test_data->spec,
@@ -334,8 +339,11 @@ test_parse_connection_spec_inet (gconstpointer data)
                          actual_in->sin_family);
     cut_assert_equal_uint(g_ntohs(expected_in->sin_port),
                           g_ntohs(actual_in->sin_port));
-    cut_assert_equal_string(cut_take_strdup(inet_ntoa(expected_in->sin_addr)),
-                            cut_take_strdup(inet_ntoa(actual_in->sin_addr)));
+    cut_assert_equal_string(
+        inet_ntop(AF_INET, &(expected_in->sin_addr),
+                  expected_address_string, sizeof(expected_address_string)),
+        inet_ntop(AF_INET, &(actual_in->sin_addr),
+                  actual_address_string, sizeof(actual_address_string)));
 }
 
 
@@ -429,7 +437,7 @@ data_address_to_spec (void)
                  test_data_free,
                  "inet no host",
                  test_data_new("inet:9999@0.0.0.0",
-                               sockaddr_in_new(9999, "0"),
+                               sockaddr_in_new(9999, NULL),
                                NULL),
                  test_data_free,
                  "unix",
@@ -451,7 +459,8 @@ data_address_to_spec (void)
                  test_data_new(NULL,
                                NULL,
                                NULL),
-                 test_data_free);
+                 test_data_free,
+                 NULL);
 }
 
 void
