@@ -45,6 +45,8 @@ void test_mi_stop (void);
 static gchar *tmp_dir;
 static guint idle_id;
 
+static MilterLogLevelFlags original_log_level;
+
 static struct smfiDesc smfilter = {
     "test milter",
     SMFI_VERSION,
@@ -65,8 +67,10 @@ static struct smfiDesc smfilter = {
 };
 
 void
-setup (void)
+cut_setup (void)
 {
+    original_log_level = milter_get_log_level();
+
     tmp_dir = g_build_filename(milter_test_get_base_dir(),
                                "tmp",
                                NULL);
@@ -78,8 +82,10 @@ setup (void)
 }
 
 void
-teardown (void)
+cut_teardown (void)
 {
+    milter_set_log_level(original_log_level);
+
     libmilter_compatible_reset();
 
     if (tmp_dir) {
@@ -147,7 +153,7 @@ test_setbacklog (void)
 void
 test_setdbg (void)
 {
-    milter_assert_equal_target_level(0);
+    milter_assert_equal_target_level(MILTER_LOG_LEVEL_NONE);
 
     milter_assert_success(smfi_setdbg(1));
     milter_assert_equal_target_level(MILTER_LOG_LEVEL_CRITICAL);
@@ -199,7 +205,7 @@ test_setconn (void)
     milter_assert_fail(smfi_setconn("unknown:/path"));
     milter_assert_success(smfi_setconn("unix:/tmp/nonexistent"));
     milter_assert_success(smfi_setconn("inet:12345"));
-    milter_assert_success(smfi_setconn("inet6:12345@localhost"));
+    milter_assert_success(smfi_setconn("inet6:12345@::1"));
 }
 
 static gboolean
