@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@cozmixng.org>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -53,6 +53,8 @@ static MilterManagerTestScenario *main_scenario;
 
 static gchar *milter_manager_program_name;
 
+static gchar *original_lang;
+
 static gchar *tmp_dir;
 
 typedef struct _EggData
@@ -162,10 +164,13 @@ egg_data_free (EggData *data)
 }
 
 void
-setup (void)
+cut_setup (void)
 {
     gchar *lt_milter_manager;
 
+    original_lang = g_strdup(g_getenv("LANG"));
+    g_setenv("LANG", "C", TRUE);
+
     scenario_dir = g_build_filename(milter_test_get_base_dir(),
                                     "fixtures",
                                     "manager",
@@ -197,7 +202,7 @@ setup (void)
 }
 
 void
-teardown (void)
+cut_teardown (void)
 {
     if (scenario_dir)
         g_free(scenario_dir);
@@ -216,6 +221,13 @@ teardown (void)
         cut_remove_path(tmp_dir, NULL);
         g_free(tmp_dir);
     }
+
+    if (original_lang) {
+        g_setenv("LANG", original_lang, TRUE);
+        g_free(original_lang);
+    } else {
+        g_unsetenv("LANG");
+    }
 }
 
 static void
@@ -306,7 +318,7 @@ wait_for_manager_ready (const gchar *spec)
             close(socket_fd);
         socket_fd = socket(domain, SOCK_STREAM, 0);
         if (socket_fd == -1)
-            break;
+            continue;
 
         if (connect(socket_fd, address, address_size) == 0) {
             errno_keep = 0;
@@ -321,8 +333,7 @@ wait_for_manager_ready (const gchar *spec)
         close(socket_fd);
 
     errno = errno_keep;
-    cut_set_message("spec: <%s>", spec);
-    cut_assert_errno();
+    cut_assert_errno(cut_message("spec: <%s>", spec));
 
     cut_assert_false(timeout_emitted);
 }
@@ -563,7 +574,9 @@ test_remove_manager_unix_socket_on_close (void)
     cut_assert_false(g_file_test(path, G_FILE_TEST_EXISTS));
 
     spec = cut_take_printf("unix:%s", path);
-    setup_egg(manager_data, "-s", spec, NULL);
+    setup_egg(manager_data,
+              "-s", spec,
+              "--config-dir", scenario_dir, NULL);
     gcut_egg_hatch(manager_egg, &error);
     gcut_assert_error(error);
 
