@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -84,10 +84,9 @@ setup_input_io (void)
     g_object_unref(input_reader);
 
     writer = milter_writer_io_channel_new(channel);
+    milter_writer_start(writer, NULL);
 
     g_io_channel_unref(channel);
-
-    milter_agent_start(MILTER_AGENT(launcher));
 }
 
 static void
@@ -109,6 +108,7 @@ setup_io (void)
 {
     setup_input_io();
     setup_output_io();
+    milter_agent_start(MILTER_AGENT(launcher), NULL);
 }
 
 void
