@@ -15,8 +15,6 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this library.  If not, see <http://www.gnu.org/licenses/>.
 
-unset LANG
-
 export BASE_DIR="`dirname $0`"
 top_dir="$BASE_DIR/.."
 top_dir="`cd $top_dir; pwd`"
@@ -62,11 +60,12 @@ MILTER_MANAGER_RUBYLIB=$MILTER_MANAGER_RUBYLIB:$ruby_dir/lib
 MILTER_MANAGER_RUBYLIB=$MILTER_MANAGER_RUBYLIB:$ruby_dir/src/toolkit/.libs
 MILTER_MANAGER_RUBYLIB=$MILTER_MANAGER_RUBYLIB:$ruby_dir/src/manager/.libs
 ruby_glib2_dir=
-if [ -f $ruby_dir/glib-0.19.3/src/glib2.so ]; then
-    ruby_glib2_dir=$ruby_dir/glib-0.19.3
-elif [ -f $ruby_dir/glib-0.16.0/src/glib2.so ]; then
-    ruby_glib2_dir=$ruby_dir/glib-0.16.0
-fi
+for dir in $(for dir in $ruby_dir/glib-*; do echo $dir; done | sort -r); do
+    if [ -f $dir/src/glib2.so ]; then
+	ruby_glib2_dir=$dir
+	break
+    fi
+done
 if [ "$ruby_glib2_dir" != "" ]; then
     MILTER_MANAGER_RUBYLIB=$MILTER_MANAGER_RUBYLIB:$ruby_glib2_dir/src/lib
     MILTER_MANAGER_RUBYLIB=$MILTER_MANAGER_RUBYLIB:$ruby_glib2_dir/src
