@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -45,6 +45,8 @@ void test_envelope_recipient_accept (void);
 void test_status (void);
 void test_state (void);
 void test_last_state (void);
+void test_macro (void);
+void test_macros_hash_table (void);
 
 static MilterServerContext *context;
 static MilterTestClient *client;
@@ -320,7 +322,7 @@ wait_ready (void)
         g_main_context_iteration(NULL, TRUE);
     }
     g_source_remove(timeout_waiting_id);
-    cut_assert_true(timeout_waiting, "timeout");
+    cut_assert_true(timeout_waiting, cut_message("timeout"));
 }
 
 static void
@@ -335,7 +337,7 @@ wait_error (void)
         g_main_context_iteration(NULL, TRUE);
     }
     g_source_remove(timeout_waiting_id);
-    cut_assert_true(timeout_waiting, "timeout");
+    cut_assert_true(timeout_waiting, cut_message("timeout"));
 }
 
 void
@@ -666,6 +668,60 @@ test_last_state (void)
                            milter_server_context_get_last_state(context));
 }
 
+void
+test_macro (void)
+{
+    MilterProtocolAgent *agent;
+
+    agent = MILTER_PROTOCOL_AGENT(context);
+    milter_protocol_agent_set_macro_context(agent, MILTER_COMMAND_CONNECT);
+
+    gcut_assert_equal_hash_table_string_string(
+        gcut_hash_table_string_string_new(NULL, NULL,
+                                          NULL),
+        milter_protocol_agent_get_available_macros(agent));
+
+    milter_protocol_agent_set_macro(agent, MILTER_COMMAND_CONNECT,
+                                    "if_name", "localhost");
+    gcut_assert_equal_hash_table_string_string(
+        gcut_hash_table_string_string_new("if_name", "localhost",
+                                          NULL),
+        milter_protocol_agent_get_available_macros(agent));
+
+    milter_protocol_agent_set_macro(agent, MILTER_COMMAND_CONNECT,
+                                    "if_name", NULL);
+    gcut_assert_equal_hash_table_string_string(
+        gcut_hash_table_string_string_new(NULL, NULL,
+                                          NULL),
+        milter_protocol_agent_get_available_macros(agent));
+}
+
+void
+test_macros_hash_table (void)
+{
+    MilterProtocolAgent *agent;
+
+    agent = MILTER_PROTOCOL_AGENT(context);
+    milter_protocol_agent_set_macro_context(agent, MILTER_COMMAND_CONNECT);
+
+    gcut_assert_equal_hash_table_string_string(
+        gcut_hash_table_string_string_new(NULL, NULL,
+                                          NULL),
+        milter_protocol_agent_get_available_macros(agent));
+
+    milter_protocol_agent_set_macros_hash_table(
+        agent, MILTER_COMMAND_CONNECT,
+        gcut_hash_table_string_string_new("if_name", "localhost",
+                                          "daemon_name", NULL,
+                                          "if_addr", "IPv6:::1",
+                                          NULL));
+    gcut_assert_equal_hash_table_string_string(
+        gcut_hash_table_string_string_new("if_name", "localhost",
+                                          "if_addr", "IPv6:::1",
+                                          NULL),
+        milter_protocol_agent_get_available_macros(agent));
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
