@@ -23,6 +23,8 @@
 
 #include <stdlib.h>
 #include <signal.h>
+#include <sys/types.h>
+#include <unistd.h>
 #include <glib/gi18n.h>
 
 #include <milter/client.h>
@@ -30,12 +32,13 @@
 static gchar *spec = NULL;
 static gboolean verbose = FALSE;
 static gboolean report_request = TRUE;
+static gboolean report_memory_profile = FALSE;
 static gboolean use_syslog = FALSE;
 static gboolean run_as_daemon = FALSE;
-static gchar *user = FALSE;
-static gchar *group = FALSE;
-static gchar *socket_group = FALSE;
-static MilterSyslogLogger *logger = NULL;
+static gchar *user = NULL;
+static gchar *group = NULL;
+static gchar *unix_socket_group = NULL;
+static guint unix_socket_mode = 0660;
 static MilterClient *client = NULL;
 
 static gboolean
@@ -72,6 +75,30 @@ parse_spec_arg (const gchar *option_name,
     return success;
 }
 
+static gboolean
+parse_unix_socket_mode (const gchar *option_name,
+                        const gchar *value,
+                        gpointer data,
+                        GError **error)
+{
+    gboolean success;
+    gchar *error_message = NULL;
+
+    success = milter_utils_parse_file_mode(value,
+                                           &unix_socket_mode,
+                                           &error_message);
+    if (!success) {
+        g_set_error(error,
+                    G_OPTION_ERROR,
+                    G_OPTION_ERROR_BAD_VALUE,
+                    _("%s"),
+                    error_message);
+        g_free(error_message);
+    }
+
+    return success;
+}
+
 static const GOptionEntry option_entries[] =
 {
     {"connection-spec", 's', 0, G_OPTION_ARG_CALLBACK, parse_spec_arg,
@@ -84,14 +111,20 @@ static const GOptionEntry option_entries[] =
     {"no-report-request", 0, G_OPTION_FLAG_NO_ARG | G_OPTION_FLAG_REVERSE,
      G_OPTION_ARG_NONE, &report_request,
      N_("Don't report request values"), NULL},
+    {"report-memory-profile", 0, G_OPTION_FLAG_NO_ARG,
+     G_OPTION_ARG_NONE, &report_memory_profile,
+     N_("Report memory profile. "
+        "Need to set MILTER_MEMORY_PROFILE=yes environment variable."), NULL},
     {"daemon", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_NONE, &run_as_daemon,
      N_("Run as a daemon"), NULL},
     {"user", 0, 0, G_OPTION_ARG_STRING, &user,
      N_("Run as USER's process (need root privilege)"), "USER"},
     {"group", 0, 0, G_OPTION_ARG_STRING, &group,
      N_("Run as GROUP's process (need root privilege)"), "GROUP"},
-    {"socket-group", 0, 0, G_OPTION_ARG_STRING, &socket_group,
+    {"unix-socket-group", 0, 0, G_OPTION_ARG_STRING, &unix_socket_group,
      N_("Change UNIX domain socket group to GROUP"), "GROUP"},
+    {"unix-socket-mode", 0, 0, G_OPTION_ARG_CALLBACK, parse_unix_socket_mode,
+     N_("Change UNIX domain socket mode to MODE (default: 0660)"), "MODE"},
     {"version", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, print_version,
      N_("Show version"), NULL},
     {NULL}
@@ -347,6 +380,66 @@ cb_error (MilterErrorEmittable *emittable, GError *error,
     g_print("ERROR: %s", error->message);
 }
 
+static guint64
+process_memory_usage_in_kb (void)
+{
+    guint64 usage = 0;
+    gchar *command_line, *result;
+
+    command_line = g_strdup_printf("ps -o rss= -p %u", getpid());
+    if (g_spawn_command_line_sync(command_line, &result, NULL, NULL, NULL)) {
+        usage = g_ascii_strtoull(result, NULL, 10);
+        g_free(result);
+    }
+    g_free(command_line);
+
+    return usage;
+}
+
+static void
+cb_maintain (MilterClient *client, gpointer user_data)
+{
+    gboolean success;
+    guint64 memory_usage_in_kb;
+    gsize n_allocates, n_zero_initializes, n_frees;
+    GString *report;
+
+    report = g_string_new("[maintain][memory] ");
+    memory_usage_in_kb = process_memory_usage_in_kb();
+    g_string_append_printf(report,
+                           "process: (%" G_GUINT64_FORMAT "KB) ",
+                           memory_usage_in_kb);
+
+    success = milter_memory_profile_get_data(&n_allocates,
+                                             &n_zero_initializes,
+                                             &n_frees);
+    if (success) {
+        guint64 n_used_in_kb;
+
+        n_used_in_kb = (n_allocates - n_frees) / 1024;
+        g_string_append_printf(report,
+                               "GLib: (%" G_GUINT64_FORMAT "KB)",
+                               n_used_in_kb);
+        if (memory_usage_in_kb == 0) {
+            g_string_append(report, "(NaN) ");
+        } else {
+            g_string_append_printf(report,
+                                   "(%.2f%%) ",
+                                   (double)n_used_in_kb / memory_usage_in_kb);
+        }
+        g_string_append_printf(report,
+                               "allocated:%" G_GSIZE_FORMAT " ",
+                               n_allocates);
+        g_string_append_printf(report,
+                               "freed:%" G_GSIZE_FORMAT "(%.2f%%)",
+                               n_frees,
+                               (double)n_frees / n_allocates * 100);
+    }
+
+    milter_statistics("%s", report->str);
+    g_string_free(report, TRUE);
+}
+
 static void
 setup_client_signals (MilterClient *client)
 {
@@ -356,6 +449,9 @@ setup_client_signals (MilterClient *client)
     CONNECT(connection_established);
     CONNECT(error);
 
+    if (report_memory_profile)
+        CONNECT(maintain);
+
 #undef CONNECT
 }
 
@@ -392,13 +488,14 @@ main (int argc, char *argv[])
 
     if (verbose)
         g_setenv("MILTER_LOG_LEVEL", "all", FALSE);
-    if (use_syslog)
-        logger = milter_syslog_logger_new("milter-test-client");
     client = milter_client_new();
+    if (use_syslog)
+        milter_client_start_syslog(client, "milter-test-client");
 
     milter_client_set_effective_user(client, user);
     milter_client_set_effective_group(client, group);
-    milter_client_set_default_unix_socket_group(client, socket_group);
+    milter_client_set_unix_socket_mode(client, unix_socket_mode);
+    milter_client_set_unix_socket_group(client, unix_socket_group);
     if (spec)
         success = milter_client_set_connection_spec(client, spec, &error);
     if (success)
@@ -419,8 +516,6 @@ main (int argc, char *argv[])
         g_error_free(error);
     }
     g_object_unref(client);
-    if (logger)
-        g_object_unref(logger);
 
     milter_client_quit();
     milter_quit();
