@@ -43,18 +43,19 @@ static gboolean verbose = FALSE;
 static gboolean output_message = FALSE;
 static gchar *spec = NULL;
 static gint negotiate_version = DEFAULT_NEGOTIATE_VERSION;
+static gint n_threads = 0;
 static gchar *connect_host = NULL;
 static struct sockaddr *connect_address = NULL;
 static socklen_t connect_address_length = 0;
 static gchar *helo_host = NULL;
 static gchar *envelope_from = NULL;
 static gchar **recipients = NULL;
-static gint current_recipient = 0;
 static gchar **body_chunks = NULL;
 static gchar *unknown_command = NULL;
 static gchar *authenticated_name = NULL;
 static gchar *authenticated_type = NULL;
 static gchar *authenticated_author = NULL;
+static gboolean use_color = FALSE;
 static MilterHeaders *option_headers = NULL;
 static gdouble connection_timeout = MILTER_SERVER_CONTEXT_DEFAULT_CONNECTION_TIMEOUT;
 static gdouble writing_timeout = MILTER_SERVER_CONTEXT_DEFAULT_WRITING_TIMEOUT;
@@ -93,6 +94,10 @@ typedef struct _ProcessData
     gchar *reply_message;
     gchar *quarantine_reason;
     MilterOption *option;
+    MilterHeaders *option_headers;
+    gint current_recipient;
+    gchar **body_chunks;
+    gint current_body_chunk;
     Message *message;
     GError *error;
 } ProcessData;
@@ -108,6 +113,7 @@ typedef struct _ProcessData
 #define CYAN_COLOR "\033[01;36m"
 #define WHITE_COLOR "\033[01;37m"
 #define NORMAL_COLOR "\033[00m"
+#define NO_COLOR ""
 
 static void
 remove_recipient (GList **recipients, const gchar *recipient)
@@ -154,11 +160,11 @@ set_macro (gpointer key, gpointer value, gpointer user_data)
 }
 
 static void
-send_recipient (MilterServerContext *context)
+send_recipient (MilterServerContext *context, ProcessData *data)
 {
     milter_server_context_envelope_recipient(context,
-                                             *(recipients + current_recipient));
-    current_recipient++;
+                                             *(recipients + data->current_recipient));
+    data->current_recipient++;
 }
 
 static void
@@ -255,6 +261,30 @@ set_macros_for_unknown (MilterProtocolAgent *agent)
 {
 }
 
+static void
+send_header (MilterServerContext *context, MilterProtocolAgent *agent, ProcessData *data)
+{
+
+    MilterHeader *header;
+
+    set_macros_for_header(agent);
+    header = milter_headers_get_nth_header(data->option_headers, 1);
+    milter_server_context_header(context, header->name, header->value);
+    milter_headers_remove(data->option_headers, header);
+}
+
+static void
+send_body_chunk (MilterServerContext *context, MilterProtocolAgent *agent, ProcessData *data)
+{
+    gchar *body_chunk;
+
+    set_macros_for_body(agent);
+
+    body_chunk = data->body_chunks[data->current_body_chunk];
+    milter_server_context_body(context, body_chunk, strlen(body_chunk));
+    data->current_body_chunk++;
+}
+
 static void
 cb_continue (MilterServerContext *context, gpointer user_data)
 {
@@ -290,93 +320,115 @@ cb_continue (MilterServerContext *context, gpointer user_data)
 
                 address.sin_family = AF_INET;
                 address.sin_port = g_htons(port);
-                inet_aton(ip_address, &(address.sin_addr));
+                inet_pton(AF_INET, ip_address, &(address.sin_addr));
                 milter_server_context_connect(context,
                                               host_name,
                                               (struct sockaddr *)(&address),
                                               sizeof(address));
             }
-            break;
+            if (!(step & MILTER_STEP_NO_REPLY_CONNECT)) {
+                break;
+            }
         }
     case MILTER_SERVER_CONTEXT_STATE_CONNECT:
         set_macros_for_helo(agent);
         if (!(step & MILTER_STEP_NO_HELO)) {
             milter_server_context_helo(context, helo_host);
-            break;
+            if (!(step & MILTER_STEP_NO_REPLY_HELO)) {
+                break;
+            }
         }
     case MILTER_SERVER_CONTEXT_STATE_HELO:
         milter_server_context_reset_message_related_data(context);
         set_macros_for_envelope_from(agent);
         if (!(step & MILTER_STEP_NO_ENVELOPE_FROM)) {
             milter_server_context_envelope_from(context, envelope_from);
-            break;
+            if (!(step & MILTER_STEP_NO_REPLY_ENVELOPE_FROM)) {
+                break;
+            }
         }
     case MILTER_SERVER_CONTEXT_STATE_ENVELOPE_FROM:
         set_macros_for_envelope_recipient(agent);
         if (!(step & MILTER_STEP_NO_ENVELOPE_RECIPIENT)) {
-            send_recipient(context);
-            break;
+            send_recipient(context, data);
+            if (!(step & MILTER_STEP_NO_REPLY_ENVELOPE_RECIPIENT)) {
+                break;
+            }
         }
     case MILTER_SERVER_CONTEXT_STATE_ENVELOPE_RECIPIENT:
         if (!(step & MILTER_STEP_NO_ENVELOPE_RECIPIENT) &&
-            *(recipients + current_recipient)) {
+            *(recipients + data->current_recipient)) {
             set_macros_for_envelope_recipient(agent);
-            send_recipient(context);
-            break;
+            send_recipient(context, data);
+            if (!(step & MILTER_STEP_NO_REPLY_ENVELOPE_RECIPIENT)) {
+                break;
+            }
         }
         if (!(step & MILTER_STEP_NO_UNKNOWN) &&
             unknown_command &&
             milter_option_get_version(data->option) >= 3) {
             set_macros_for_unknown(agent);
             milter_server_context_unknown(context, unknown_command);
-            break;
+            if (!(step & MILTER_STEP_NO_REPLY_UNKNOWN)) {
+                break;
+            }
         }
     case MILTER_SERVER_CONTEXT_STATE_UNKNOWN:
         set_macros_for_data(agent);
         if (!(step & MILTER_STEP_NO_DATA) &&
             milter_option_get_version(data->option) >= 4) {
             milter_server_context_data(context);
-            break;
+            if (!(step & MILTER_STEP_NO_REPLY_DATA)) {
+                break;
+            }
         }
     case MILTER_SERVER_CONTEXT_STATE_DATA:
         set_macros_for_header(agent);
         if (!(step & MILTER_STEP_NO_HEADERS)) {
             MilterHeader *header;
-            header = milter_headers_get_nth_header(option_headers, 1);
+            header = milter_headers_get_nth_header(data->option_headers, 1);
             milter_server_context_header(context, header->name, header->value);
-            milter_headers_remove(option_headers, header);
-            break;
+            milter_headers_remove(data->option_headers, header);
+            if (!(step & MILTER_STEP_NO_REPLY_HEADER)) {
+                break;
+            }
         }
     case MILTER_SERVER_CONTEXT_STATE_HEADER:
         if (!(step & MILTER_STEP_NO_HEADERS) &&
-            milter_headers_length(option_headers) > 0) {
-            MilterHeader *header;
+            milter_headers_length(data->option_headers) > 0) {
+            send_header(context, agent, data);
+            if (!(step & MILTER_STEP_NO_REPLY_HEADER)) {
+                break;
+            }
 
-            set_macros_for_header(agent);
-            header = milter_headers_get_nth_header(option_headers, 1);
-            milter_server_context_header(context, header->name, header->value);
-            milter_headers_remove(option_headers, header);
-            break;
+            while (milter_headers_length(data->option_headers) > 0) {
+                send_header(context, agent, data);
+            }
         }
         set_macros_for_end_of_header(agent);
         if (!(step & MILTER_STEP_NO_END_OF_HEADER)) {
             milter_server_context_end_of_header(context);
-            break;
+            if (!(step & MILTER_STEP_NO_REPLY_END_OF_HEADER)) {
+                break;
+            }
         }
     case MILTER_SERVER_CONTEXT_STATE_END_OF_HEADER:
-        set_macros_for_body(agent);
         if (!(step & MILTER_STEP_NO_BODY)) {
-            milter_server_context_body(context, *body_chunks, strlen(*body_chunks));
-            body_chunks++;
-            break;
+            send_body_chunk(context, agent, data);
+            if (!(step & MILTER_STEP_NO_REPLY_BODY)) {
+                break;
+            }
         }
     case MILTER_SERVER_CONTEXT_STATE_BODY:
         if (!(step & MILTER_STEP_NO_BODY) &&
-            *body_chunks) {
-            set_macros_for_body(agent);
-            milter_server_context_body(context, *body_chunks, strlen(*body_chunks));
-            body_chunks++;
-            break;
+            data->body_chunks[data->current_body_chunk]) {
+            send_body_chunk(context, agent, data);
+            if (!(step & MILTER_STEP_NO_REPLY_BODY)) {
+                break;
+            }
+            while(data->body_chunks[data->current_body_chunk]) {
+                send_body_chunk(context, agent, data);
+            }
         }
         set_macros_for_end_of_message(agent);
         milter_server_context_end_of_message(context, NULL, 0);
@@ -420,7 +472,7 @@ cb_temporary_failure (MilterServerContext *context, gpointer user_data)
     switch (state) {
     case MILTER_SERVER_CONTEXT_STATE_ENVELOPE_RECIPIENT:
         remove_recipient(&(data->message->recipients),
-                         *(recipients + current_recipient - 1));
+                         *(recipients + data->current_recipient - 1));
         if (data->message->recipients) {
             cb_continue(context, user_data);
             break;
@@ -443,7 +495,7 @@ cb_reject (MilterServerContext *context, gpointer user_data)
     switch (state) {
     case MILTER_SERVER_CONTEXT_STATE_ENVELOPE_RECIPIENT:
         remove_recipient(&(data->message->recipients),
-                         *(recipients + current_recipient - 1));
+                         *(recipients + data->current_recipient - 1));
         if (data->message->recipients) {
             cb_continue(context, user_data);
             break;
@@ -658,8 +710,9 @@ cb_shutdown (MilterServerContext *context, gpointer user_data)
 static void
 cb_skip (MilterServerContext *context, gpointer user_data)
 {
-    while (*body_chunks)
-        body_chunks++;
+    ProcessData *data = user_data;
+    while (data->body_chunks[data->current_body_chunk])
+        data->current_body_chunk++;
 
     cb_continue(context, user_data);
 }
@@ -1151,6 +1204,30 @@ parse_mail_file_arg (const gchar *option_name,
     return TRUE;
 }
 
+static gboolean
+parse_color_arg (const gchar *option_name, const gchar *value,
+                 gpointer data, GError **error)
+{
+    if (value == NULL ||
+        g_utf8_collate(value, "yes") == 0 ||
+        g_utf8_collate(value, "true") == 0) {
+        use_color = TRUE;
+    } else if (g_utf8_collate(value, "no") == 0 ||
+               g_utf8_collate(value, "false") == 0) {
+        use_color = FALSE;
+    } else if (g_utf8_collate(value, "auto") == 0) {
+        use_color = milter_utils_guess_console_color_usability();
+    } else {
+        g_set_error(error,
+                    G_OPTION_ERROR,
+                    G_OPTION_ERROR_BAD_VALUE,
+                    _("Invalid color value: %s"), value);
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
 static const GOptionEntry option_entries[] =
 {
     {"connection-spec", 's', 0, G_OPTION_ARG_CALLBACK, parse_spec_arg,
@@ -1197,6 +1274,9 @@ static const GOptionEntry option_entries[] =
     {"output-message", 0,
      G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_NONE, &output_message,
      N_("Output modified message"), NULL},
+    {"color", 0, G_OPTION_FLAG_OPTIONAL_ARG,
+     G_OPTION_ARG_CALLBACK, parse_color_arg,
+     N_("Output modified message with colors"), "[yes|true|no|false|auto]"},
     {"connection-timeout", 0, 0, G_OPTION_ARG_DOUBLE, &connection_timeout,
      N_("Timeout after SECONDS seconds on connecting to a milter."), "SECONDS"},
     {"reading-timeout", 0, 0, G_OPTION_ARG_DOUBLE, &reading_timeout,
@@ -1205,6 +1285,8 @@ static const GOptionEntry option_entries[] =
      N_("Timeout after SECONDS seconds on writing a command."), "SECONDS"},
     {"end-of-message-timeout", 0, 0, G_OPTION_ARG_DOUBLE, &writing_timeout,
      N_("Timeout after SECONDS seconds on end-of-message command."), "SECONDS"},
+    {"threads", 't', 0, G_OPTION_ARG_INT, &n_threads,
+     N_("Create N threads."), "N"},
     {"verbose", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_NONE, &verbose,
      N_("Be verbose"), NULL},
     {"version", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, print_version,
@@ -1260,6 +1342,10 @@ init_process_data (ProcessData *data)
     data->success = TRUE;
     data->quarantine_reason = NULL;
     data->option = NULL;
+    data->option_headers = milter_headers_copy(option_headers);
+    data->body_chunks = g_strdupv(body_chunks);
+    data->current_body_chunk = 0;
+    data->current_recipient = 0;
     data->reply_code = 0;
     data->reply_extended_code = NULL;
     data->reply_message = NULL;
@@ -1274,6 +1360,10 @@ free_process_data (ProcessData *data)
         g_timer_destroy(data->timer);
     if (data->option)
         g_object_unref(data->option);
+    if (data->option_headers)
+        g_object_unref(data->option_headers);
+    if (data->body_chunks)
+        g_strfreev(data->body_chunks);
     if (data->reply_extended_code)
         g_free(data->reply_extended_code);
     if (data->reply_message)
@@ -1293,6 +1383,7 @@ pre_option_parse (GOptionContext *option_context,
                   GError **error)
 {
     option_headers = milter_headers_new();
+    use_color = milter_utils_guess_console_color_usability();
 
     return TRUE;
 }
@@ -1336,28 +1427,42 @@ static void
 print_header (const gchar *prefix, MilterHeader *header)
 {
     gchar **value_lines, **first_line;
-    const gchar *color = NORMAL_COLOR;
+    const gchar *color_starter = NO_COLOR;
+    const gchar *color_finisher = NO_COLOR;
 
-    if (g_str_has_prefix(prefix, "+"))
-        color = GREEN_COLOR;
-    else if (g_str_has_prefix(prefix, "-"))
-        color = RED_COLOR;
+    if (use_color) {
+        if (g_str_has_prefix(prefix, "+"))
+            color_starter = GREEN_COLOR;
+        else if (g_str_has_prefix(prefix, "-"))
+            color_starter = RED_COLOR;
+        else
+            color_starter = NORMAL_COLOR;
+        color_finisher = NORMAL_COLOR;
+    }
 
     value_lines = g_strsplit(header->value, "\n", -1);
     first_line = value_lines;
 
     if (!*value_lines) {
-        g_printf("%s%s %s:%s\n", color, prefix, header->name, NORMAL_COLOR);
+        g_printf("%s%s %s:%s\n",
+                 color_starter,
+                 prefix, header->name,
+                 color_finisher);
         g_strfreev(value_lines);
         return;
     }
 
-    g_printf("%s%s %s: %s%s\n", color, prefix, header->name, *value_lines,
-             NORMAL_COLOR);
+    g_printf("%s%s %s: %s%s\n",
+             color_starter,
+             prefix, header->name, *value_lines,
+             color_finisher);
     value_lines++;
 
     while (*value_lines) {
-        g_printf("%s%s %s%s\n", color, prefix, *value_lines, NORMAL_COLOR);
+        g_printf("%s%s %s%s\n",
+                 color_starter,
+                 prefix, *value_lines,
+                 color_finisher);
         value_lines++;
     }
 
@@ -1577,15 +1682,30 @@ start_process (MilterServerContext *context, ProcessData *process_data)
     return TRUE;
 }
 
+static gpointer
+test_server_thread (gpointer data)
+{
+    gboolean success;
+    ProcessData *process_data = data;
+    MilterServerContext *context;
+
+    context = milter_server_context_new();
+    setup_context(context, process_data);
+
+    success = start_process(context, process_data);
+
+    g_object_unref(context);
+
+    return GINT_TO_POINTER(success);
+}
+
 int
 main (int argc, char *argv[])
 {
     gboolean success = TRUE;
-    MilterServerContext *context;
     GError *error = NULL;
     GOptionContext *option_context;
     GOptionGroup *main_group;
-    ProcessData process_data;
 
     milter_init();
     milter_server_init();
@@ -1608,14 +1728,32 @@ main (int argc, char *argv[])
     if (verbose)
         g_setenv("MILTER_LOG_LEVEL", "all", FALSE);
 
-    context = milter_server_context_new();
-    init_process_data(&process_data);
-    setup_context(context, &process_data);
+    if (n_threads > 0) {
+        GThread **threads;
+        ProcessData *process_data;
+        gint i;
 
-    success = start_process(context, &process_data);
+        threads = g_new0(GThread*, n_threads);
+        process_data = g_new0(ProcessData, n_threads);
+        for (i = 0; i < n_threads; i++) {
+            init_process_data(&process_data[i]);
+            threads[i] = g_thread_create(test_server_thread, &process_data[i], TRUE, NULL);
+        }
+
+        for (i = 0; i < n_threads; i++) {
+            success &= GPOINTER_TO_INT(g_thread_join(threads[i]));
+            free_process_data(&process_data[i]);
+        }
+
+        g_free(process_data);
+        g_free(threads);
+    } else {
+        ProcessData process_data;
+        init_process_data(&process_data);
+        success = GPOINTER_TO_INT(test_server_thread(&process_data));
+        free_process_data(&process_data);
+    }
 
-    g_object_unref(context);
-    free_process_data(&process_data);
 
     milter_server_quit();
     milter_quit();
